@@ -31,7 +31,7 @@ import pygame
 import sge
 
 
-__all__ = ['Object', 'Mouse', '_PygameProjectionSprite']
+__all__ = ['Object', 'Mouse']
 
 
 class Object(object):
@@ -120,7 +120,7 @@ class Object(object):
 
        Depending on the game, a useful strategy to boost performance can
        be to exclude an object from collision detection while it is
-       outside the view.  If you do this, you likely also to set
+       outside the view.  If you do this, you likely want to set
        :attr:`active` to :const:`False` as well so that the object
        doesn't move in undesireable ways (e.g. through walls).
 
@@ -320,9 +320,6 @@ class Object(object):
         self._x = value
         self._update_collision_areas()
 
-        # Cause the Pygame sprite to make itself dirty
-        self._pygame_sprite.rect = pygame.Rect(0, 0, 1, 1)
-
     @property
     def y(self):
         return self._y
@@ -332,28 +329,6 @@ class Object(object):
         self._y = value
         self._update_collision_areas()
 
-        # Cause the Pygame sprite to make itself dirty
-        self._pygame_sprite.rect = pygame.Rect(0, 0, 1, 1)
-
-    @property
-    def z(self):
-        return self._z
-
-    @z.setter
-    def z(self, value):
-        self._z = value
-        if self._pygame_sprite in sge.game._pygame_sprites:
-            self._pygame_sprite.kill()
-            sge.game._pygame_sprites.add(self._pygame_sprite, layer=self._z)
-
-        # Cause the Pygame sprite to make itself dirty
-        self._pygame_sprite.rect = pygame.Rect(0, 0, 1, 1)
-
-        # Make the mouse have a higher Z-axis value
-        if ("mouse" in sge.game.objects and
-                sge.game.objects["mouse"].z <= value):
-            sge.game.objects["mouse"]._z = value + 1
-
     @property
     def sprite(self):
         return self._sprite
@@ -381,7 +356,7 @@ class Object(object):
                 position = i
                 break
 
-        if self._tangible:
+        if self.tangible:
             if position is None:
                 sge.game._colliders.append(weakref.ref(self))
         else:
@@ -746,7 +721,7 @@ class Object(object):
 
         self._x = x
         self._y = y
-        self._z = z
+        self.z = z
         self._xvelocity = 0
         self._yvelocity = 0
         self._move_direction = 0
@@ -772,10 +747,6 @@ class Object(object):
 
         self._rect = pygame.Rect(self.bbox_x, self.bbox_y, self.bbox_width,
                                  self.bbox_height)
-        if self.id != "mouse":
-            self._pygame_sprite = _PygameSprite(self)
-        else:
-            self._pygame_sprite = _FakePygameSprite(self)
 
         self.z = z
 
@@ -815,7 +786,7 @@ class Object(object):
           :const:`None`, :attr:`y` will be used.
 
         """
-        if self.tangible:
+        if self.tangible and other.tangible:
             room = sge.game.current_room
             others = []
             collisions = []
@@ -911,9 +882,7 @@ class Object(object):
 
     def destroy(self):
         """Destroy the object."""
-        sge.game._background_changed = True
         self.event_destroy()
-        self._pygame_sprite.kill()
         objects = sge.game.objects.copy()
         if self.id in objects:
             del objects[self.id]
@@ -1570,25 +1539,25 @@ class Object(object):
         # Get the width of the sprite when scaled rotated.
         return int(self.sprite._get_image(
             self.image_index, self.image_xscale, self.image_yscale,
-            self.image_rotation).get_width() / sge.game._xscale)
+            self.image_rotation).get_width())
 
     def _get_normal_image_width(self):
         # Get the width of the sprite without scaling and rotation.
         return int(self.sprite._get_image(
             self.image_index, self.image_xscale,
-            self.image_yscale).get_width() / sge.game._xscale)
+            self.image_yscale).get_width())
 
     def _get_image_height(self):
         # Get the height of the sprite when scaled rotated.
         return int(self.sprite._get_image(
             self.image_index, self.image_xscale, self.image_yscale,
-            self.image_rotation).get_height() / sge.game._xscale)
+            self.image_rotation).get_height())
 
     def _get_normal_image_height(self):
         # Get the height of the sprite without scaling and rotation.
         return int(self.sprite._get_image(
             self.image_index, self.image_xscale,
-            self.image_yscale).get_height() / sge.game._xscale)
+            self.image_yscale).get_height())
 
     def _get_origin_offset(self):
         # Return the amount to offset the origin as (x, y).
@@ -1671,67 +1640,39 @@ class Mouse(Object):
 
     @property
     def x(self):
-        if sge.game.current_room is not None:
-            mouse_x = self.mouse_x / sge.game._xscale
-            mouse_y = self.mouse_y / sge.game._yscale
+        if (sge.game.current_room is not None and
+                (not sge.game.grab_input or sge.game.mouse.visible)):
+            mouse_x = sge.mouse.get_x()
+            mouse_y = sge.mouse.get_y()
             for view in sge.game.current_room.views:
-                if (view.xport <= mouse_x <= view.xport + view.width and
-                        view.yport <= mouse_y <= view.yport + view.height):
-                    # We save this value so that if the mouse is in none of
-                    # the views, the last known position in a view is used.
-                    self._x = (mouse_x - view.xport) + view.x
-                    break
+                if (view.xport <= mouse_x < view.xport + view.wport and
+                        view.yport <= mouse_y < view.yport + view.hport):
+                    return ((mouse_x - view.xport) *
+                            (view.width / view.wport) + view.x)
 
-            return self._x
-        else:
-            return 0
+        return -1
 
     @x.setter
     def x(self, value):
-        rel_x = (value - self.x) * sge.game._xscale
-        self.mouse_x += rel_x
-
-        self.xprevious = self._x
-        self._x = value
-        self._bbox_left = value + self.bbox_x
-        self._bbox_right = self.bbox_left + self.bbox_width
-
-        # Cause the Pygame sprite to make itself dirty
-        self._pygame_sprite.rect = pygame.Rect(0, 0, 1, 1)
-
-        pygame.mouse.set_pos(self.mouse_x, self.mouse_y)
+        pass
 
     @property
     def y(self):
-        if sge.game.current_room is not None:
-            mouse_x = self.mouse_x / sge.game._xscale
-            mouse_y = self.mouse_y / sge.game._yscale
+        if (sge.game.current_room is not None and
+                (not sge.game.grab_input or sge.game.mouse.visible)):
+            mouse_x = sge.mouse.get_x()
+            mouse_y = sge.mouse.get_y()
             for view in sge.game.current_room.views:
-                if (view.xport <= mouse_x <= view.xport + view.width and
-                        view.yport <= mouse_y <= view.yport + view.height):
-                    # We save this value so that if the mouse is in none of
-                    # the views, the last known position in a view is used.
-                    self._y = (mouse_y - view.yport) + view.y
-                    break
+                if (view.xport <= mouse_x <= view.xport + view.wport and
+                        view.yport <= mouse_y <= view.yport + view.hport):
+                    return ((mouse_y - view.yport) *
+                            (view.height / view.hport) + view.y)
 
-            return self._y
-        else:
-            return 0
+        return -1
 
     @y.setter
     def y(self, value):
-        rel_y = (value - self.y) * sge.game._yscale
-        self.mouse_y += rel_y
-
-        self.yprevious = self._y
-        self._y = value
-        self._bbox_top = value + self.bbox_y
-        self._bbox_bottom = self.bbox_top + self.bbox_height
-
-        # Cause the Pygame sprite to make itself dirty
-        self._pygame_sprite.rect = pygame.Rect(0, 0, 1, 1)
-
-        pygame.mouse.set_pos(self.mouse_x, self.mouse_y)
+        pass
 
     @property
     def sprite(self):
@@ -1755,12 +1696,31 @@ class Mouse(Object):
         self._visible = value
         self.set_cursor()
 
+    @property
+    def tangible(self):
+        if self.x != -1 and self.y != -1:
+            return self._tangible
+        else:
+            return False
+
+    @tangible.setter
+    def tangible(self, value):
+        self._tangible = value
+        position = None
+        for i in range(len(sge.game._colliders)):
+            if sge.game._colliders[i]() is self:
+                position = i
+                break
+
+        if self.tangible:
+            if position is None:
+                sge.game._colliders.append(weakref.ref(self))
+        else:
+            if position is not None:
+                del sge.game._colliders[position]
+
     def __init__(self):
         self._visible = True
-        self.mouse_x, self.mouse_y = pygame.mouse.get_pos()
-        self.mouse_xprevious = self.mouse_x
-        self.mouse_yprevious = self.mouse_y
-        self.previous_speeds = []
 
         super(Mouse, self).__init__(0, 0, 0, ID='mouse')
 
@@ -1780,42 +1740,9 @@ class Mouse(Object):
         sge.game.event_mouse_collision_bottom(other)
 
     def _update(self, time_passed, delta_mult):
-        self.update_speed(time_passed)
         super(Mouse, self)._update(time_passed, delta_mult)
         self._update_collision_areas()
 
-    def update_speed(self, time_passed):
-        # Update the speed variables.  ``time_passed`` is the number of
-        # milliseconds since the last speed update.
-        self.previous_speeds.insert(0, (self.mouse_x - self.mouse_xprevious,
-                                        self.mouse_y - self.mouse_yprevious,
-                                        time_passed))
-        time = 0
-        num_steps = 0
-        total_xvelocity = 0
-        total_yvelocity = 0
-
-        for speed in self.previous_speeds:
-            time += speed[2]
-            if time <= 250:
-                num_steps += 1
-                total_xvelocity += speed[0]
-                total_yvelocity += speed[1]
-
-        self.previous_speeds = self.previous_speeds[:num_steps]
-
-        if num_steps > 0:
-            self.xvelocity = total_xvelocity / num_steps
-            self.yvelocity = total_yvelocity / num_steps
-        else:
-            self.xvelocity = 0
-            self.yvelocity = 0
-
-        self.mouse_xprevious = self.mouse_x
-        self.mouse_yprevious = self.mouse_y
-        self.xprevious = self.x
-        self.yprevious = self.y
-
     def set_cursor(self):
         # Set the mouse cursor and visibility state.
         if not sge.game.grab_input:
@@ -1826,225 +1753,13 @@ class Mouse(Object):
     def project_cursor(self):
         if (not sge.game.grab_input and self.visible and
                 self.sprite is not None):
-            x = (self.mouse_x / sge.game._xscale) - self.image_origin_x
-            y = (self.mouse_y / sge.game._yscale) - self.image_origin_y
             img = self.sprite._get_image(
                 self.image_index, self.image_xscale, self.image_yscale, 
                 self.image_rotation, self.image_alpha, self.image_blend)
+            x = sge.mouse.get_x()
+            y = sge.mouse.get_y()
 
-            sge.game._window_projections.append((img, x, y, None))
-
-    def _reset(self):
-        # Reset the mouse back to its original state, but NOT position.
-        self.z = self._start_z
-        self.sprite = self._start_sprite
-        self.visible = self._start_visible
-        self.detects_collisions = self._start_detects_collisions
-        self.bbox_x = self._start_bbox_x
-        self.bbox_y = self._start_bbox_y
-        self.bbox_width = self._start_bbox_width
-        self.bbox_height = self._start_bbox_height
-        self.collision_ellipse = self._start_collision_ellipse
-        self.collision_precise = self._start_collision_precise
-
-
-class _PygameSprite(pygame.sprite.DirtySprite):
-
-    # Handles drawing in this implementation.
-    #
-    # Scaling is handled transparently in the update method, which is
-    # always called before drawing.  Everything else is the
-    # responsibility of Object, including animation (the current
-    # frame is grabbed from the _image attribute of the parent object).
-
-    def __init__(self, parent, *groups):
-        # See pygame.sprite.DirtySprite.__init__.__doc__.  ``parent``
-        # is a Object object that this object belongs to.
-        super(_PygameSprite, self).__init__(*groups)
-        self.parent = weakref.ref(parent)
-        self.image = pygame.Surface((1, 1))
-        self.image.set_colorkey((0, 0, 0))
-        self.rect = self.image.get_rect()
-
-    def update(self):
-        if self.parent() is not None:
-            parent = self.parent()
-            if parent.sprite is not None:
-                new_image = parent.sprite._get_image(
-                    parent.image_index, parent.image_xscale,
-                    parent.image_yscale, parent.image_rotation,
-                    parent.image_alpha, parent.image_blend)
-                if self.image is not new_image:
-                    self.must_redraw = False
-                    self.image = new_image
-                    self.dirty = 1
-
-                if self.visible != parent.visible:
-                    self.visible = int(parent.visible)
-                    self.dirty = 1
-
-                width = parent._get_image_width()
-                normal_width = parent._get_normal_image_width()
-                height = parent._get_image_height()
-                normal_height = parent._get_normal_image_height()
-                x_offset = (width - normal_width) / 2
-                y_offset = (height - normal_height) / 2
-
-                self.update_rect(parent.x, parent.y, parent.z, parent.sprite,
-                                 parent.image_origin_x + x_offset,
-                                 parent.image_origin_y + y_offset)
-            else:
-                self.image = pygame.Surface((1, 1))
-                self.image.set_colorkey((0, 0, 0))
-                self.dirty = 1
-        else:
-            self.kill()
-
-    def update_rect(self, x, y, z, sprite, origin_x, origin_y):
-        # Update the rect of this Pygame sprite, based on the SGE sprite
-        # and coordinates given.  This involves creating "proxy"
-        # one-time sprites for multiple views if necessary.
-        views = sge.game.current_room.views
-
-        if (len(views) == 1 and views[0].xport == 0 and views[0].yport == 0 and
-                views[0].width == sge.game.width and
-                views[0].height == sge.game.height):
-            # There is only one view that takes up the whole screen, so
-            # we don't need to worry about it.
-            x = x - views[0].x - origin_x
-            y = y - views[0].y - origin_y
-            new_rect = self.image.get_rect()
-            new_rect.left = round(x * sge.game._xscale) + sge.game._x
-            new_rect.top = round(y * sge.game._yscale) + sge.game._y
-
-            if self.rect != new_rect:
-                self.rect = new_rect
-                self.dirty = 1
-        else:
-            # There is something more complicated.  Have to account for
-            # the possibility of edges or multiple appearances.
-            original_used = False
-            self.dirty = 1
-            real_x = x
-            real_y = y
-            for view in views:
-                x = real_x - view.x - origin_x + view.xport
-                y = real_y - view.y - origin_y + view.yport
-                w = sprite.width
-                h = sprite.height
-                new_rect = self.image.get_rect()
-                new_rect.left = round(x * sge.game._xscale) + sge.game._x
-                new_rect.top = round(y * sge.game._yscale) + sge.game._y
-                inside_view = (x >= view.xport and
-                               x + w <= view.xport + view.width and
-                               y >= view.yport and
-                               y + h <= view.yport + view.height)
-                within_view = (x + w > view.xport and
-                               x < view.xport + view.width and
-                               y + h > view.yport and
-                               y < view.yport + view.height)
-
-                if not original_used and inside_view:
-                    original_used = True
-                    if self.rect == new_rect:
-                        self.dirty = 0
-                    else:
-                        self.rect = new_rect
-                elif within_view:
-                    if inside_view:
-                        img = self.image
-                        rect = new_rect
-                    else:
-                        # Make a cut-off version of the sprite and
-                        # adjust the rect accordingly.
-                        if x < view.xport:
-                            cut_x = view.xport - x
-                            x = view.xport
-                            w -= cut_x
-                        else:
-                            cut_x = 0
-
-                        if x + w > view.xport + view.width:
-                            w -= (x + w) - (view.xport + view.width)
-
-                        if y < view.yport:
-                            cut_y = view.yport - y
-                            y = view.yport
-                            h -= cut_y
-                        else:
-                            cut_y = 0
-
-                        if y + h > view.yport + view.height:
-                            h -= (y + h) - (view.yport + view.height)
-
-                        x = sge.game._x + round(x * sge.game._xscale)
-                        y = sge.game._y + round(y * sge.game._yscale)
-                        cut_x *= sge.game._xscale
-                        cut_y *= sge.game._yscale
-                        w = round(w * sge.game._xscale)
-                        h = round(h * sge.game._yscale)
-                        rect = pygame.Rect(x, y, w, h)
-                        cut_rect = pygame.Rect(cut_x, cut_y, w, h)
-                        img = self.image.subsurface(cut_rect)
-
-                    # Create proxy one-time sprite
-                    proxy = _PygameOneTimeSprite(img, rect)
-                    sge.game._pygame_sprites.add(proxy, layer=z)
-
-            if not original_used:
-                self.image = pygame.Surface((1, 1))
-                self.image.set_colorkey((0, 0, 0))
-
-
-class _PygameProjectionSprite(_PygameSprite):
-
-    # Special Pygame sprite used for room projections.
-
-    def __init__(self, x, y, z, sprite, image_index=0):
-        pygame.sprite.DirtySprite.__init__(self)
-        self.dirty = 1
-        self.x = x
-        self.y = y
-        self.z = z
-        if isinstance(sprite, sge.Sprite):
-            self.sprite = sprite
-        else:
-            self.sprite = sge.game.sprites[sprite]
-        self.image_index = image_index
-        self.image = self.sprite._get_image(image_index)
-        self.rect = self.image.get_rect()
-
-    def update(self):
-        if self.dirty:
-            self.image = self.sprite._get_image(self.image_index)
-            self.update_rect(self.x, self.y, self.z, self.sprite,
-                             self.sprite.origin_x, self.sprite.origin_y)
-        else:
-            self.kill()
-
-
-class _FakePygameSprite(_PygameSprite):
-
-    # Fake Pygame sprite for "use" by the mouse.
-
-    def update(self):
-        pass
-
-    def update_rect(self):
-        pass
-
-
-class _PygameOneTimeSprite(pygame.sprite.DirtySprite):
-
-    # A regular DirtySprite that only displays once, and then destroys
-    # itself.
-
-    def __init__(self, image, rect, *groups):
-        super(_PygameOneTimeSprite, self).__init__(*groups)
-        self.image = image
-        self.rect = rect
-        self.dirty = 1
-
-    def update(self):
-        if not self.dirty:
-            self.kill()
+            if x is not None and y is not None:
+                x -= self.image_origin_x
+                y -= self.image_origin_y
+                sge.game._window_projections.append((img, x, y, None))
