@@ -116,6 +116,7 @@ class Room(object):
         self.background_y = background_y
         self._destroyed = False
         self._new_objects = []
+        self._projections = []
 
         self._alarms = {}
 
@@ -216,10 +217,8 @@ class Room(object):
                     objects_by_class[cls] = objects
             self.objects_by_class = objects_by_class
 
-            if self is sge.game.current_room:
-                sge.game._pygame_sprites.add(obj._pygame_sprite, layer=obj.z)
-                if self._started:
-                    obj.event_create()
+            if self is sge.game.current_room and self._started:
+                obj.event_create()
             else:
                 self._new_objects.append(obj)
 
@@ -241,15 +240,7 @@ class Room(object):
         method behaves in the same way that :meth:`sge.Room.start` does.
 
         """
-        for sprite in sge.game._pygame_sprites:
-            sprite.kill()
-
-        self._limit_views()
         sge.game.current_room = self
-        sge.game._background_changed = True
-
-        for obj in self.objects:
-            sge.game._pygame_sprites.add(obj._pygame_sprite, layer=obj.z)
 
         if not self._has_started:
             self._start_objects = self.objects[:]
@@ -520,26 +511,13 @@ class Room(object):
         more information.
 
         """
-        if blend_mode is not None and blend_mode & sge.BLEND_SCREEN:
-            w = "Screen blend mode not supported. Normal blending used instead."
-            warnings.warn(w)
+        if not isinstance(sprite, sge.Sprite):
+            sprite = sge.game.sprites[sprite]
 
-        pygame_flags = {
-            sge.BLEND_RGBA_ADD: pygame.BLEND_RGBA_ADD,
-            sge.BLEND_RGBA_SUBTRACT: pygame.BLEND_RGBA_SUB,
-            sge.BLEND_RGBA_MULTIPLY: pygame.BLEND_RGBA_MULT,
-            sge.BLEND_RGBA_MINIMUM: pygame.BLEND_RGBA_MIN,
-            sge.BLEND_RGBA_MAXIMUM: pygame.BLEND_RGBA_MAX,
-            sge.BLEND_RGB_ADD: pygame.BLEND_RGB_ADD,
-            sge.BLEND_RGB_SUBTRACT: pygame.BLEND_RGB_SUB,
-            sge.BLEND_RGB_MULTIPLY: pygame.BLEND_RGB_MULT,
-            sge.BLEND_RGB_MINIMUM: pygame.BLEND_RGB_MIN,
-            sge.BLEND_RGB_MAXIMUM: pygame.BLEND_RGB_MAX
-            }.setdefault(blend_mode, 0)
-
-        p = sge._PygameProjectionSprite(x, y, z, sprite, image)
-        p.blendmode = pygame_flags
-        sge.game._pygame_sprites.add(p, layer=z)
+        img = sprite._get_image(image)
+        x -= sprite.origin_x
+        y -= sprite.origin_y
+        self._projections.append((img, x, y, z, blend_mode))
 
     def project_text(self, font, text, x, y, z, width=None, height=None,
                     color=sge.Color("black"), halign=sge.ALIGN_LEFT,
@@ -916,19 +894,6 @@ class Room(object):
         """
         pass
 
-    def _limit_views(self):
-        # Prevent the views from moving outside of the room.
-        for view in self.views:
-            if view.x < 0:
-                view._x = 0
-            elif view.x + view.width > self.width:
-                view._x = self.width - view.width
-
-            if view.y < 0:
-                view._y = 0
-            elif view.y + view.height > self.height:
-                view._y = self.height - view.height
-
     def _reset(self):
         # Reset the room to its original state.
         self._started = False
