@@ -367,15 +367,6 @@ support anti-aliasing.
 :data:`sge.BLEND_RGBA_SCREEN` and :data:`sge.BLEND_RGB_SCREEN` are
 unsupported. If one of these blend modes is attempted, normal blending
 will be used instead.
-
-Speed Improvements
-------------------
-
-The Pygame SGE supports hardware rendering, which can improve
-performance in some cases.  It is not enabled by default.  To enable it,
-set :data:`sge.hardware_rendering` to :const:`True`.  The benefit of
-hardware acceleration is usually negligible, which is why it is disabled
-by default.
 """
 
 from __future__ import division
@@ -383,7 +374,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.11.0"
+__version__ = "0.12"
 
 import sys
 import os
@@ -507,7 +498,7 @@ from sge.Background import Background
 from sge.Font import Font
 from sge.Sound import Sound
 from sge.Music import Music
-from sge.Object import Object, Mouse, _PygameProjectionSprite
+from sge.Object import Object, Mouse
 from sge.Room import Room
 from sge.View import View
 from sge import collision, input, joystick, keyboard, mouse
@@ -551,11 +542,12 @@ hardware_rendering = False
 
 
 def _scale(surface, width, height):
-    # Scale the given surface to the given width and height, taking the
-    # scale factor of the screen into account.
-    width = int(round(width * game._xscale))
-    height = int(round(height * game._yscale))
+    # Scale the given surface to the given width and height.
+    if surface.get_width() == width and surface.get_height() == height:
+        return surface.copy()
 
+    width = int(round(width))
+    height = int(round(height))
     if game.scale_smooth:
         try:
             new_surf = pygame.transform.smoothscale(surface, (width, height))
@@ -565,3 +557,25 @@ def _scale(surface, width, height):
         new_surf = pygame.transform.scale(surface, (width, height))
 
     return new_surf
+
+
+def _get_blend_flags(blend_mode):
+    # Return the appropriate Pygame flags for the given blend mode.
+    if blend_mode is not None and blend_mode & BLEND_SCREEN:
+        w = "Screen blend mode not supported. Normal blending used instead."
+        warnings.warn(w)
+
+    pygame_flags = {
+        BLEND_RGBA_ADD: pygame.BLEND_RGBA_ADD,
+        BLEND_RGBA_SUBTRACT: pygame.BLEND_RGBA_SUB,
+        BLEND_RGBA_MULTIPLY: pygame.BLEND_RGBA_MULT,
+        BLEND_RGBA_MINIMUM: pygame.BLEND_RGBA_MIN,
+        BLEND_RGBA_MAXIMUM: pygame.BLEND_RGBA_MAX,
+        BLEND_RGB_ADD: pygame.BLEND_RGB_ADD,
+        BLEND_RGB_SUBTRACT: pygame.BLEND_RGB_SUB,
+        BLEND_RGB_MULTIPLY: pygame.BLEND_RGB_MULT,
+        BLEND_RGB_MINIMUM: pygame.BLEND_RGB_MIN,
+        BLEND_RGB_MAXIMUM: pygame.BLEND_RGB_MAX
+        }.get(blend_mode, 0)
+
+    return pygame_flags
